@@ -53,6 +53,12 @@ def gen_fakehw():
     # not covered by any driver package
     t.add_device('pci', 'grey', None, ['modalias', 'pci:vDEADBEEFd00'], [])
     t.add_device('ssb', 'yellow', None, [], ['MODALIAS', 'pci:vDEADBEEFd00'])
+    # covered by stracciatella.deb / universe
+    t.add_device('pci', 'orange', None, ['modalias', 'pci:v98761234d00sv00000001sd00bc00sc00i00'], [])
+    # covered by neapolitan.deb / restricted
+    t.add_device('pci', 'purple', None, ['modalias', 'pci:v67891234d00sv00000001sd00bc00sc00i00'], [])
+    # covered by tuttifrutti.deb / multiverse
+    t.add_device('usb', 'aubergine', None, ['modalias', 'usb:v1234dABCDsv01sd02bc00sc01i05'], [])
 
     return t
 
@@ -74,6 +80,20 @@ def gen_fakearchive():
     a.create_deb('nvidia-old', dependencies={'Depends': 'xorg-video-abi-3'},
                  extra_tags={'Modaliases': 'nv(pci:v000010DEd000010C3sv*sd*bc03sc*i*, pci:v000010DEd000010C2sv*sd*bc03sc*i*,)'})
 
+    # Free package in universe
+    a.create_deb('stracciatella',
+                 component='universe',
+                 extra_tags={'Modaliases': 'stracciatella(pci:v98761234d*sv*sd*bc*sc*i*, pci:v0000BEEFd*sv*sd*bc*sc*i*)'})
+
+    # Non-free packages
+    a.create_deb('neapolitan',
+                 component='restricted',
+                 extra_tags={'Modaliases': 'neapolitan(pci:v67891234d*sv*sd*bc*sc*i*, pci:v0000BEEFd*sv*sd*bc*sc*i*)'})
+
+    a.create_deb('tuttifrutti',
+                 component='multiverse',
+                 extra_tags={'Modaliases': 'tuttifrutti(usb:v1234dABCDsv*sd*bc00sc*i*, pci:v0000BEEFd*sv*sd*bc*sc*i00)'})
+
     # packages not covered by modalises, for testing detection plugins
     a.create_deb('special')
     a.create_deb('picky')
@@ -102,6 +122,7 @@ class DetectTest(unittest.TestCase):
         # no custom detection plugins by default
         self.plugin_dir = tempfile.mkdtemp()
         os.environ['UBUNTU_DRIVERS_DETECT_DIR'] = self.plugin_dir
+        os.environ['UBUNTU_DRIVERS_SYS_DIR'] = self.umockdev.get_sys_dir()
 
     def tearDown(self):
         shutil.rmtree(self.plugin_dir)
@@ -126,11 +147,14 @@ class DetectTest(unittest.TestCase):
     def test_system_modalises_fake(self):
         '''system_modaliases() for fake sysfs'''
 
-        res = UbuntuDrivers.detect.system_modaliases()
+        res = UbuntuDrivers.detect.system_modaliases(self.umockdev.get_sys_dir())
         self.assertEqual(set(res), set(['pci:v00001234d00sv00000001sd00bc00sc00i00',
             'pci:vDEADBEEFd00', 'usb:v9876dABCDsv01sd02bc00sc01i05',
+            'usb:v1234dABCDsv01sd02bc00sc01i05',
+            'pci:v98761234d00sv00000001sd00bc00sc00i00',
+            'pci:v67891234d00sv00000001sd00bc00sc00i00',
             modalias_nv]))
-        self.assertEqual(res['pci:vDEADBEEFd00'], '/sys/devices/grey')
+        self.assertTrue(res['pci:vDEADBEEFd00'].endswith('/sys/devices/grey'))
 
     def test_system_driver_packages_performance(self):
         '''system_driver_packages() performance for a lot of modaliases'''
@@ -141,7 +165,7 @@ class DetectTest(unittest.TestCase):
             self.umockdev.add_device('usb', 'usbdev%i' % i, None, ['modalias', 'usb:s%04X' % i], [])
 
         start = resource.getrusage(resource.RUSAGE_SELF)
-        UbuntuDrivers.detect.system_driver_packages()
+        UbuntuDrivers.detect.system_driver_packages(sys_path=self.umockdev.get_sys_dir())
         stop = resource.getrusage(resource.RUSAGE_SELF)
 
         sec = (stop.ru_utime + stop.ru_stime) - (start.ru_utime + start.ru_stime)
@@ -175,16 +199,30 @@ class DetectTest(unittest.TestCase):
                                dependencies={'Depends': 'xorg-video-abi-3 | xorg-video-abi-4'},
                                extra_tags={'Modaliases': 'nv(pci:v000010DEd000010C3sv*sd*bc03sc*i*)'})
             chroot.add_repository(archive.path, True, False)
+
+            # Overwrite sources list generate by aptdaemon testsuite to add
+            # options to apt and ignore unsigned repository
+            sources_list = os.path.join(chroot.path,'etc/apt/sources.list')
+            with open(sources_list, 'w') as f:
+                f.write(archive.apt_source)
+
             cache = apt.Cache(rootdir=chroot.path)
-            res = UbuntuDrivers.detect.system_driver_packages(cache)
+            cache.update()
+            cache.open()
+            res = UbuntuDrivers.detect.system_driver_packages(cache, sys_path=self.umockdev.get_sys_dir())
         finally:
             chroot.remove()
         self.assertEqual(set(res), set(['chocolate', 'vanilla', 'nvidia-current',
                                         'nvidia-current-updates', 'nvidia-123',
-                                        'nvidia-34']))
+                                        'nvidia-34',
+                                        'neapolitan',
+                                        'tuttifrutti',
+                                        'stracciatella',
+                                        ]))
+
         self.assertEqual(res['vanilla']['modalias'], 'pci:v00001234d00sv00000001sd00bc00sc00i00')
         self.assertTrue(res['vanilla']['syspath'].endswith('/devices/white'))
-        self.assertFalse(res['vanilla']['from_distro'])
+        self.assertTrue(res['vanilla']['from_distro'])
         self.assertTrue(res['vanilla']['free'])
         self.assertFalse('vendor' in res['vanilla'])
         self.assertFalse('model' in res['vanilla'])
@@ -215,6 +253,44 @@ class DetectTest(unittest.TestCase):
         self.assertEqual(res['nvidia-34']['modalias'], modalias_nv)
         self.assertEqual(res['nvidia-34']['recommended'], False)
 
+        self.assertFalse(res['neapolitan']['free'])
+
+    def test_system_gpgpu_driver_packages_chroot(self):
+        '''system_driver_packages() for test package repository'''
+
+        chroot = aptdaemon.test.Chroot()
+        try:
+            chroot.setup()
+            chroot.add_test_repository()
+            archive = gen_fakearchive()
+            # older applicable driver which is not the recommended one
+            archive.create_deb('nvidia-driver-390', dependencies={'Depends': 'xorg-video-abi-4'},
+                               extra_tags={'Modaliases': 'nv(pci:v000010DEd000010C3sv*sd*bc03sc*i*)'})
+            # -updates driver which also should not be recommended
+            archive.create_deb('nvidia-driver-410', dependencies={'Depends': 'xorg-video-abi-4'},
+                               extra_tags={'Modaliases': 'nv(pci:v000010DEd000010C3sv*sd*bc03sc*i*)'})
+            # driver package which supports multiple ABIs
+            archive.create_deb('nvidia-340',
+                               dependencies={'Depends': 'xorg-video-abi-3 | xorg-video-abi-4'},
+                               extra_tags={'Modaliases': 'nv(pci:v000010DEd000010C3sv*sd*bc03sc*i*)'})
+            archive.create_deb('nvidia-headless-no-dkms-410',
+                               dependencies={'Depends': 'xorg-video-abi-3 | xorg-video-abi-4'},
+                               extra_tags={})
+            archive.create_deb('nvidia-headless-no-dkms-390',
+                               dependencies={'Depends': 'xorg-video-abi-3 | xorg-video-abi-4'},
+                               extra_tags={})
+
+            chroot.add_repository(archive.path, True, False)
+            cache = apt.Cache(rootdir=chroot.path)
+            res = UbuntuDrivers.detect.system_gpgpu_driver_packages(cache, sys_path=self.umockdev.get_sys_dir())
+        finally:
+            chroot.remove()
+        self.assertTrue('nvidia-driver-410' in res)
+        packages = UbuntuDrivers.detect.gpgpu_install_filter(res, 'nvidia')
+        self.assertEqual(set(packages), set(['nvidia-driver-410']))
+        driver = list(packages.keys())[0]
+        self.assertEqual(packages[driver].get('metapackage'), 'nvidia-headless-no-dkms-410')
+
     def test_system_driver_packages_bad_encoding(self):
         '''system_driver_packages() with badly encoded Packages index'''
 
@@ -237,11 +313,14 @@ Description: broken \xEB encoding
 ''')
             chroot.add_repository(archive.path, True, False)
             cache = apt.Cache(rootdir=chroot.path)
-            res = UbuntuDrivers.detect.system_driver_packages(cache)
+            res = UbuntuDrivers.detect.system_driver_packages(cache, sys_path=self.umockdev.get_sys_dir())
         finally:
             chroot.remove()
 
-        self.assertEqual(set(res), set(['chocolate', 'vanilla', 'nvidia-current']))
+        self.assertEqual(
+            set(res),
+            set(['chocolate', 'vanilla', 'nvidia-current',
+                 'neapolitan', 'tuttifrutti', 'stracciatella']))
 
     def test_system_driver_packages_detect_plugins(self):
         '''system_driver_packages() includes custom detection plugins'''
@@ -249,7 +328,7 @@ Description: broken \xEB encoding
         with open(os.path.join(self.plugin_dir, 'extra.py'), 'w') as f:
             f.write('def detect(apt): return ["coreutils", "no_such_package"]\n')
 
-        res = UbuntuDrivers.detect.system_driver_packages() 
+        res = UbuntuDrivers.detect.system_driver_packages(sys_path=self.umockdev.get_sys_dir()) 
         self.assertTrue('coreutils' in res, list(res.keys()))
         self.assertEqual(res['coreutils'], {'free': True, 'from_distro': True, 'plugin': 'extra.py'})
 
@@ -273,42 +352,50 @@ Description: broken \xEB encoding
                                extra_tags={'Modaliases': 'nv(pci:v000010DEd000010C3sv*sd*bc03sc*i*)'})
             chroot.add_repository(archive.path, True, False)
             cache = apt.Cache(rootdir=chroot.path)
-            res = UbuntuDrivers.detect.system_device_drivers(cache)
+            res = UbuntuDrivers.detect.system_device_drivers(cache, sys_path=self.umockdev.get_sys_dir())
         finally:
             chroot.remove()
 
         white = '/sys/devices/white'
         black = '/sys/devices/black'
         graphics = '/sys/devices/graphics'
-        self.assertEqual(len(res), 3)  # the three devices above
+        self.assertEqual(len(res), 6)  # the three devices above + 3 fake devices
+        self.assertEqual(
+            set([os.path.basename(d) for d in res]),
+            set(['white', 'purple', 'aubergine', 'orange', 'graphics', 'black']))
 
-        self.assertEqual(res[white], 
+
+        white_dict = [value for key, value in res.items() if key.endswith(white)][0]
+        black_dict = [value for key, value in res.items() if key.endswith(black)][0]
+        graphics_dict = [value for key, value in res.items() if key.endswith(graphics)][0]
+
+        self.assertEqual(white_dict,
                          {'modalias': 'pci:v00001234d00sv00000001sd00bc00sc00i00',
                           'drivers': {'vanilla': {'free': True, 'from_distro': False}}
                          })
 
-        self.assertEqual(res[black], 
+        self.assertEqual(black_dict,
                          {'modalias': 'usb:v9876dABCDsv01sd02bc00sc01i05',
                           'drivers': {'chocolate': {'free': True, 'from_distro': False}}
                          })
 
-        self.assertEqual(res[graphics]['modalias'], modalias_nv)
-        self.assertTrue('nvidia' in res[graphics]['vendor'].lower())
-        self.assertTrue('GeForce' in res[graphics]['model'])
+        self.assertEqual(graphics_dict['modalias'], modalias_nv)
+        self.assertTrue('nvidia' in graphics_dict['vendor'].lower())
+        self.assertTrue('GeForce' in graphics_dict['model'])
 
         # should contain nouveau driver; note that free is True here because
         # these come from the fake archive
-        self.assertEqual(res[graphics]['drivers']['nvidia-current'],
+        self.assertEqual(graphics_dict['drivers']['nvidia-current'],
                          {'free': True, 'from_distro': False, 'recommended': True})
-        self.assertEqual(res[graphics]['drivers']['nvidia-current-updates'],
+        self.assertEqual(graphics_dict['drivers']['nvidia-current-updates'],
                          {'free': True, 'from_distro': False, 'recommended': False})
-        self.assertEqual(res[graphics]['drivers']['nvidia-123'],
+        self.assertEqual(graphics_dict['drivers']['nvidia-123'],
                           {'free': True, 'from_distro': False, 'recommended': False})
-        self.assertEqual(res[graphics]['drivers']['nvidia-experimental'],
+        self.assertEqual(graphics_dict['drivers']['nvidia-experimental'],
                          {'free': True, 'from_distro': False, 'recommended': False})
-        self.assertEqual(res[graphics]['drivers']['xserver-xorg-video-nouveau'],
+        self.assertEqual(graphics_dict['drivers']['xserver-xorg-video-nouveau'],
                          {'free': True, 'from_distro': True, 'recommended': False, 'builtin': True})
-        self.assertEqual(len(res[graphics]['drivers']), 5, list(res[graphics]['drivers'].keys()))
+        self.assertEqual(len(graphics_dict['drivers']), 5, list(graphics_dict['drivers'].keys()))
 
     def test_system_device_drivers_detect_plugins(self):
         '''system_device_drivers() includes custom detection plugins'''
@@ -316,7 +403,7 @@ Description: broken \xEB encoding
         with open(os.path.join(self.plugin_dir, 'extra.py'), 'w') as f:
             f.write('def detect(apt): return ["coreutils", "no_such_package"]\n')
 
-        res = UbuntuDrivers.detect.system_device_drivers()
+        res = UbuntuDrivers.detect.system_device_drivers(sys_path=self.umockdev.get_sys_dir())
         self.assertTrue('extra.py' in res, list(res.keys()))
         self.assertEqual(res['extra.py'],
                          {'drivers': {'coreutils': {'free': True, 'from_distro': True}}})
@@ -345,17 +432,18 @@ exec /sbin/modinfo "$@"
             orig_path = os.environ['PATH']
             os.environ['PATH'] = '%s:%s' % (chroot.path, os.environ['PATH'])
 
-            res = UbuntuDrivers.detect.system_device_drivers(cache)
+            res = UbuntuDrivers.detect.system_device_drivers(cache, sys_path=self.umockdev.get_sys_dir())
         finally:
             chroot.remove()
             os.environ['PATH'] = orig_path
 
         graphics = '/sys/devices/graphics'
-        self.assertEqual(res[graphics]['modalias'], modalias_nv)
-        self.assertTrue(res[graphics]['manual_install'])
+        graphics_dict = [value for key, value in res.items() if key.endswith(graphics)][0]
+        self.assertEqual(graphics_dict['modalias'], modalias_nv)
+        self.assertTrue(graphics_dict['manual_install'])
 
         # should still show the drivers
-        self.assertGreater(len(res[graphics]['drivers']), 1)
+        self.assertGreater(len(graphics_dict['drivers']), 1)
 
     def test_auto_install_filter(self):
         '''auto_install_filter()'''
@@ -377,6 +465,97 @@ exec /sbin/modinfo "$@"
         self.assertEqual(set(UbuntuDrivers.detect.auto_install_filter(pkgs)),
                          set(['bcmwl-kernel-source', 'nvidia-173']))
 
+
+    def test_gpgpu_install_filter(self):
+        '''gpgpu_install_filter()'''
+
+        #gpgpu driver[:version][,driver[:version]]
+        self.assertEqual(UbuntuDrivers.detect.gpgpu_install_filter({}, 'nvidia'), {})
+
+        pkgs = {'nvidia-driver-390': {'recommended': True},
+                'nvidia-driver-410': {},
+                'nvidia-driver-340': {'recommended': False}}
+
+        # Nothing is specified, we return the recommended driver
+        self.assertEqual(set(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, 'nvidia')),
+            set(['nvidia-driver-390']))
+
+        # We specify that we want nvidia 410
+        pkgs = {'nvidia-driver-390': {'recommended': True},
+                'nvidia-driver-410': {},
+                'nvidia-driver-340': {'recommended': False}}
+        self.assertEqual(set(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, '410')),
+                         set(['nvidia-driver-410']))
+
+        self.assertEqual(set(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, 'nvidia:410')),
+                         set(['nvidia-driver-410']))
+
+        # Now with multiple drivers (to be implemented in the future)
+        self.assertEqual(set(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, 'nvidia:410,amdgpu:284')),
+                         set(['nvidia-driver-410']))
+
+        # Specify the same nvidia driver twice, just to break things
+        self.assertEqual(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, 'nvidia:410,nvidia:390'),
+                         {})
+
+        # More incorrect values
+        self.assertEqual(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, 'nv:410'), {})
+
+        self.assertEqual(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, ':410'), {})
+
+        self.assertEqual(UbuntuDrivers.detect.gpgpu_install_filter(pkgs, 'nvidia-driver:410'), {})
+
+    def test_system_driver_packages_freeonly(self):
+        '''system_driver_packages() returns only free packages'''
+
+        chroot = aptdaemon.test.Chroot()
+        try:
+            chroot.setup()
+            archive = gen_fakearchive()
+            chroot.add_repository(archive.path, True, False)
+
+            # Overwrite sources list generate by aptdaemon testsuite to add
+            # options to apt and ignore unsigned repository
+            sources_list = os.path.join(chroot.path,'etc/apt/sources.list')
+            with open(sources_list, 'w') as f:
+                f.write(archive.apt_source)
+
+            cache = apt.Cache(rootdir=chroot.path)
+            cache.update()
+            cache.open()
+
+            res = set(UbuntuDrivers.detect.system_driver_packages(cache, sys_path=self.umockdev.get_sys_dir(), freeonly=True))
+        finally:
+            chroot.remove()
+
+        self.assertEqual(res, set(['stracciatella', 'vanilla', 'chocolate', 'nvidia-current']))
+
+    def test_system_device_drivers_freeonly(self):
+        '''system_device_drivers() returns only devices with free drivers'''
+
+        chroot = aptdaemon.test.Chroot()
+        try:
+            chroot.setup()
+            archive = gen_fakearchive()
+            chroot.add_repository(archive.path, True, False)
+
+            # Overwrite sources list generate by aptdaemon testsuite to add
+            # options to apt and ignore unsigned repository
+            sources_list = os.path.join(chroot.path,'etc/apt/sources.list')
+            with open(sources_list, 'w') as f:
+                f.write(archive.apt_source)
+
+            cache = apt.Cache(rootdir=chroot.path)
+            cache.update()
+            cache.open()
+
+            res = UbuntuDrivers.detect.system_device_drivers(cache, sys_path=self.umockdev.get_sys_dir(), freeonly=True)
+        finally:
+            chroot.remove()
+        self.assertEqual(
+            set([os.path.basename(d) for d in res]),
+            set(['black', 'white', 'graphics', 'orange']))
+
     def test_detect_plugin_packages(self):
         '''detect_plugin_packages()'''
 
@@ -563,6 +742,7 @@ APT::Get::AllowUnauthenticated "true";
         '''Create a fake sysfs'''
 
         self.umockdev = gen_fakehw()
+        os.environ['UBUNTU_DRIVERS_SYS_DIR'] = self.umockdev.get_sys_dir()
 
     def tearDown(self):
         # some tests install this package
@@ -580,7 +760,8 @@ APT::Get::AllowUnauthenticated "true";
         out, err = ud.communicate()
         self.assertEqual(err, '')
         self.assertEqual(set(out.splitlines()), 
-                set(['vanilla', 'chocolate', 'bcmwl-kernel-source', 'nvidia-current']))
+                set(['vanilla', 'chocolate', 'bcmwl-kernel-source', 'nvidia-current',
+                     'stracciatella', 'tuttifrutti', 'neapolitan']))
         self.assertEqual(ud.returncode, 0)
 
     def test_list_detect_plugins(self):
@@ -599,22 +780,8 @@ APT::Get::AllowUnauthenticated "true";
         self.assertEqual(err, '')
         self.assertEqual(set(out.splitlines()), 
                 set(['vanilla', 'chocolate', 'bcmwl-kernel-source',
-                     'nvidia-current', 'special', 'picky']))
-        self.assertEqual(ud.returncode, 0)
-
-    def test_list_system(self):
-        '''ubuntu-drivers list for fake sysfs and system apt'''
-
-        env = os.environ.copy()
-        del env['APT_CONFIG']
-
-        ud = subprocess.Popen([self.tool_path, 'list'],
-                universal_newlines=True, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE, env=env)
-        out, err = ud.communicate()
-        # real system packages should not match our fake modalises
-        self.assertEqual(err, '')
-        self.assertEqual(out, '\n')
+                     'nvidia-current', 'special', 'picky',
+                     'stracciatella', 'tuttifrutti', 'neapolitan']))
         self.assertEqual(ud.returncode, 0)
 
     def test_devices_chroot(self):
@@ -657,27 +824,12 @@ APT::Get::AllowUnauthenticated "true";
         self.assertTrue('special - third-party free' in out, out)
         self.assertEqual(ud.returncode, 0)
 
-    def test_devices_system(self):
-        '''ubuntu-drivers devices for fake sysfs and system apt'''
-
-        env = os.environ.copy()
-        del env['APT_CONFIG']
-
-        ud = subprocess.Popen([self.tool_path, 'devices'],
-                universal_newlines=True, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE, env=env)
-        out, err = ud.communicate()
-        # real system packages should not match our fake modalises
-        self.assertEqual(err, '')
-        self.assertEqual(out, '')
-        self.assertEqual(ud.returncode, 0)
-
     def test_auto_install_chroot(self):
-        '''ubuntu-drivers autoinstall for fake sysfs and chroot'''
+        '''ubuntu-drivers install for fake sysfs and chroot'''
 
-        ud = subprocess.Popen([self.tool_path, 'autoinstall'],
+        ud = subprocess.Popen([self.tool_path, 'install'],
                 universal_newlines=True, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE)
+                stderr=subprocess.PIPE, env=os.environ)
         out, err = ud.communicate()
         self.assertEqual(err, '')
         self.assertTrue('bcmwl-kernel-source' in out, out)
@@ -686,21 +838,21 @@ APT::Get::AllowUnauthenticated "true";
         self.assertEqual(ud.returncode, 0)
 
         # now all packages should be installed, so it should not do anything
-        ud = subprocess.Popen([self.tool_path, 'autoinstall'],
+        ud = subprocess.Popen([self.tool_path, 'install'],
                 universal_newlines=True, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE)
+                stderr=subprocess.PIPE, env=os.environ)
         out, err = ud.communicate()
         self.assertEqual(err, '')
         self.assertFalse('bcmwl-kernel-source' in out, out)
         self.assertEqual(ud.returncode, 0)
 
     def test_auto_install_packagelist(self):
-        '''ubuntu-drivers autoinstall package list creation'''
+        '''ubuntu-drivers install package list creation'''
 
         listfile = os.path.join(self.chroot.path, 'pkgs')
         self.addCleanup(os.unlink, listfile)
 
-        ud = subprocess.Popen([self.tool_path, 'autoinstall', '--package-list', listfile],
+        ud = subprocess.Popen([self.tool_path, 'install', '--package-list', listfile],
                 universal_newlines=True, stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE)
         out, err = ud.communicate()
@@ -710,21 +862,6 @@ APT::Get::AllowUnauthenticated "true";
         with open(listfile) as f:
             self.assertEqual(f.read(), 'bcmwl-kernel-source\n')
 
-    def test_auto_install_system(self):
-        '''ubuntu-drivers autoinstall for fake sysfs and system apt'''
-
-        env = os.environ.copy()
-        del env['APT_CONFIG']
-
-        ud = subprocess.Popen([self.tool_path, 'autoinstall'],
-                universal_newlines=True, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE, env=env)
-        out, err = ud.communicate()
-        self.assertEqual(err, '')
-        # real system packages should not match our fake modalises
-        self.assertTrue('No drivers found' in out)
-        self.assertEqual(ud.returncode, 0)
-
     def test_debug(self):
         '''ubuntu-drivers debug'''
 
@@ -778,6 +915,7 @@ class KernelDectionTest(unittest.TestCase):
         # no custom detection plugins by default
         self.plugin_dir = tempfile.mkdtemp()
         os.environ['UBUNTU_DRIVERS_DETECT_DIR'] = self.plugin_dir
+        os.environ['UBUNTU_DRIVERS_SYS_DIR'] = self.umockdev.get_sys_dir()
 
     def tearDown(self):
         shutil.rmtree(self.plugin_dir)
